#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

# Web page: https://github.com/LinuxBeaver?tab=repositories
# Binaries_for_windows_and_linux (gegl_command.py_compatible_plugins) https://github.com/LinuxBeaver/Special_Gimp_Plugins_for_GEGL_command.py/releases/tag/gegl_command.py_compatible_plugins

# Change Log

# Rel 1.0: Initial release: 23/11/2023 LinuxBeaver [ aka contrast_]: 5 plugins (scheduled 24): Glossy Balloon, Custom Bevel, Sharp Bevel, Metallic, and Action Lines
# Rel 1.1:
#------------------------------------------------------------------------------------------------------
from gimpfu import *
from sys import argv, platform
from ctypes import *
import random
import math
import sys, os

def rgb_to_hex(rgb_color):
    r = rgb_color[0]
    g = rgb_color[1]
    b = rgb_color[2]
    return '#%02x%02x%02x' % (r,g,b)
#-------------------------------------Glossy_Balloon-----------------------------------------------------   

def gegl_glossy_balloon(image, layer, gaus,hue,lightness,opacityall):


    gegl_graph_string="lb:script-glossy-balloon gaus=%f hue=%f lightness=%f opacityall=%f" % (gaus, hue, lightness, opacityall)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
    "python_fu_gegl_glossy_balloon",
    "GEGL to Python. Works on the active layer.\nRequires installation of an additional binary file.",
    "Glossy Balloon GEGL Text Styling Plugin In Python (Special fork)",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/LinuxBeaver/gegl-glossy-balloon...",             #Menu path
    "*",
    [
    (PF_FLOAT, "gaus","Balloonification (default:6.0, 0.5 20.0):", 6.0),   
    (PF_FLOAT, "hue","Rotate Color (default:0, -180 180.0):", 0),
    (PF_FLOAT, "lightness","Lightness (default:-7, -15.0 15.0):", -7),
    (PF_FLOAT, "opacityall","Internal Opacity (default:0.0, 1.0 5.0):", 3.0),

    ],
    [],
    gegl_glossy_balloon)

#-------------------------------------Custom_Bevel-----------------------------------------------------   

def gegl_custom_bevel(image, layer, opacity, blendmode, azimuth, elevation, depth, size, alphapercentile, gaus, box, mcb):


    if blendmode==0:
        blend_mode= "Hardlight"
    if blendmode==1:
        blend_mode= "Multiply"
    if blendmode==2:
        blend_mode= "ColorDodge"
    if blendmode==3:
        blend_mode= "Plus"
    if blendmode==4:
        blend_mode= "Darken"
    if blendmode==5:
        blend_mode= "Lighten"



    gegl_graph_string="lb:script-custom-bevel opacity=%f blendmode=%s azimuth=%f elevation=%f depth=%f size=%f alphapercentile=%f gaus=%f box=%f mcb=%f" % (opacity, blend_mode, azimuth, elevation, depth, size, alphapercentile, gaus, box, mcb)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
    "python_fu_gegl_custom_bevel",
    "GEGL to Python. Works on the active layer.\nRequires installation of an additional binary file.",
    "Custom Bevel GEGL Text Styling Plugin In Python (Special fork)",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/LinuxBeaver/gegl-custom-bevel...",             #Menu path
    "*",
    [
    (PF_FLOAT, "opacity","Internal opacity (default:1.0, 3.5 10.0):", 3.5),
    (PF_OPTION,  "blend_mode", "Blendmode:", 0, ["Hardlight","Multiply","ColorDodge","Plus","Darken","Lighten"]),
    (PF_FLOAT, "azimuth","Azimuth (default:67, 30 90.0):", 67),
    (PF_FLOAT, "elevation","Elevation (default:25, 7 90):", 25),
    (PF_FLOAT, "depth","Depth (default:24, 1 100):", 24),
    (PF_FLOAT, "size","Internal Median Radius (default:1, 0 15):", 1),
    (PF_FLOAT, "alphapercentile","Internal Median AP (default:68, 0 100):", 68),
    (PF_FLOAT, "gaus","Internal Gaussian Blur (default:2, 0 9):", 2),
    (PF_FLOAT, "box","Internal Box Blur (default:0, 0 6):", 0),
    (PF_FLOAT, "mcb","Smooth (default:0, 0 6):", 1),

    ],
    [],
    gegl_custom_bevel)

#-------------------------------------Sharp_Bevel-----------------------------------------------------   

def gegl_sharp_bevel(image, layer, blendmode,  azimuth, elevation, depth,  size,  transvalue, bevelcontrol, smooth):

    if blendmode==0:
        blend_mode= "hardlight"
    if blendmode==1:
        blend_mode= "multiply"
    if blendmode==2:
        blend_mode= "colordodge"
    if blendmode==3:
        blend_mode= "plus"
    if blendmode==4:
        blend_mode= "darken"
    if blendmode==5:
        blend_mode= "lighten"
    if blendmode==6:
        blend_mode= "overlay"


    gegl_graph_string="lb:script-sharpbevel blendmode=%s azimuth=%f elevation=%f depth=%f size=%f transvalue=%f bevelcontrol=%f smooth=%f" % (blend_mode, azimuth, elevation, depth, size, transvalue, bevelcontrol, smooth)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
    "python_fu_gegl_sharp_bevel",
    "GEGL to Python. Works on the active layer.\nRequires installation of an additional binary file.",
    "Sharp Bevel GEGL Text Styling Plugin In Python (Special fork)",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/LinuxBeaver/gegl-sharp-bevel...",             #Menu path
    "*",
    [
    (PF_OPTION,  "blend_mode", "Blendmode:", 0, ["Hardlight","Multiply","ColorDodge","Plus","Darken","Lighten","Overlay"]),
    (PF_FLOAT, "azimuth","Azimuth (default:67, 20 90.0):", 67),
    (PF_FLOAT, "elevation","Elevation (default:25, 7 140):", 25),
    (PF_FLOAT, "depth","Depth (default:24, 8 100):", 24),
    (PF_FLOAT, "size","Size (default:3, 1 9):", 3),
    (PF_FLOAT, "transvalue","Remove Black Artifact (default:0.05, 0.00 0.03):", 0.05),
    (PF_FLOAT, "bevelcontrol","Make Surface Flat (default:1, 1 6):", 1),
    (PF_FLOAT, "smooth","Smooth (default:9, 1 20):", 9),


    ],
    [],
    gegl_sharp_bevel)
#-------------------------------------Metallic-----------------------------------------------------

def gegl_metallic(image, layer, solar1, solar2, solar3, light, smooth, value):


    gegl_graph_string="lb:script-metallic solar1=%f solar2=%f solar3=%f light=%f smooth=%f value=%f" % (solar1, solar2, solar3, light, smooth, value)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
    "python_fu_gegl_metallic",
    "GEGL to Python. Works on the active layer.\nRequires installation of an additional binary file.",
    "Metallic GEGL Plugin In Python (Special fork)",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/LinuxBeaver/gegl-metallic...",             #Menu path
    "*",
    [
    (PF_FLOAT, "solar1","Solarization of Red Channel (default:2.7, 0.0 4.0):", 2.7),
    (PF_FLOAT, "solar2","Solarization of Green Channel (default:2.8, 2.2 4.0):", 2.8),
    (PF_FLOAT, "solar3","Solarization of Blue Channel (default:2.1, 0.0 4.0):", 2.1),
    (PF_FLOAT, "light","Lightness (default:0, -10.0 10.0):", 0),      
    (PF_FLOAT, "smooth","Smooth (default:2, 0 8):", 2),
    (PF_FLOAT, "value","Invert Original Image (default:0.0, 1.0 5.0):", 0),

    ],
    [],
    gegl_metallic)

#-------------------------------------Action_Lines-----------------------------------------------------

def gegl_action_lines(image, layer, movex,movey,color_al,radius,lines, seed_al, opacity):

    color_to_gegl_al=rgb_to_hex(color_al)
    if seed_al==0:
        seed=0
    if seed_al==1:
        seed=random.randrange(0,4294967296)      

    gegl_graph_string="lb:action-lines movex=%f movey=%f color=%s radius=%d lines=%d seed=%s opacity=%f" % (movex, movey, color_to_gegl_al, radius, lines, seed, opacity)
    pdb.python_fu_gegl_graph(image, layer, gegl_graph_string)

register(
    "python_fu_gegl_action_lines",
    "GEGL to Python. Works on the active layer.\nRequires installation of an additional binary file.",
    "Action Lines GEGL Plugin In Python",
    "Auto Author",
    "Auto Author",
    "Auto Generated Dated",
    "<Image>/GEGL as Python/LinuxBeaver/gegl-action_lines...",             #Menu path
    "*",
    [
    (PF_FLOAT, "movex","Move Horizontal (default:0.500, 0.0 0.80):", 0.500),
    (PF_FLOAT, "movey","Move Vertical (default:0.500, 0.0 0.80):", 0.500),
    (PF_COLOR, "color_al",         "Color:", (0, 0, 0)),
    (PF_SPINNER, "radius","Radius of Lines (default:3200, 800..10000):", 3200, (800, 10000, 3200)),
    (PF_SPINNER, "lines","Amount of Lines (default:700, 380..1024):", 700, (380, 1024, 700)),
    (PF_OPTION,  "seed_al", "Seed:", 0, ["Seed=0","Random seed"]),
    (PF_FLOAT, "opacity","Opacity of Lines (default:2.0, 0.6 3.0):", 2.0),

    ],
    [],
    gegl_action_lines)
#-------------------------------------

main()